#include "RTE_Components.h"
#include CMSIS_device_header
#include "stm32f2xx_hal.h"              // Keil::Device:STM32Cube HAL:Common
#include "cmsis_os2.h"

#include <stdio.h>
#include <stdint.h>

#include "lwip/dhcp.h"
#include "lwip/init.h"
#include "lwip/netif.h"
#include "lwip/priv/tcp_priv.h"
#include "netif/etharp.h"
#include "ethernetif.h"
#include "lwip/apps/httpd.h"

#include "Board_GLCD.h"
#include "GLCD_Config.h"

#include "Board_LED.h"

extern GLCD_FONT GLCD_Font_6x8;
extern GLCD_FONT GLCD_Font_16x24;


#define NET_TICK_INTERVAL_MS  10

static void net_init (void);
static void net_periodic (uint32_t localtime);
static void net_timer_tick (void const *arg);

struct netif netif;
uint32_t LocalTime = 0;
uint8_t  LEDs = 0;

/**
  * Override default HAL_GetTick function
  */
uint32_t HAL_GetTick (void) {
  static uint32_t ticks = 0U;
         uint32_t i;

  if (osKernelGetState () == osKernelRunning) {
    return ((uint32_t)osKernelGetTickCount ());
  }

  /* If Kernel is not running wait approximately 1 ms then increment 
     and return auxiliary tick counter value */
  for (i = (SystemCoreClock >> 14U); i > 0U; i--) {
    __NOP(); __NOP(); __NOP(); __NOP(); __NOP(); __NOP();
    __NOP(); __NOP(); __NOP(); __NOP(); __NOP(); __NOP();
  }
  return ++ticks;
}

/// System Clock Configuration
void SystemClock_Config(void) {
  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;

  /* Enable HSE Oscillator and activate PLL with HSE as source */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 25;
  RCC_OscInitStruct.PLL.PLLN = 240;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = 5;
  HAL_RCC_OscConfig(&RCC_OscInitStruct);

  /* Select PLL as system clock source and configure the HCLK, PCLK1 and PCLK2
     clocks dividers */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_HCLK |
                                RCC_CLOCKTYPE_PCLK1  | RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;
  HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_3);
}

static void net_timer_tick (void const *arg) {
  /* System tick timer callback function */
  LocalTime += NET_TICK_INTERVAL_MS;
}

static void net_create_timer (void) {
  osTimerId_t id;
  id = osTimerNew ((osTimerFunc_t)&net_timer_tick, osTimerPeriodic, NULL, NULL);
  osTimerStart (id, NET_TICK_INTERVAL_MS);
}

static void net_init (void) {
  struct ip4_addr ipaddr;
  struct ip4_addr netmask;
  struct ip4_addr gw;

  lwip_init ();

#if LWIP_DHCP
  ipaddr.addr = 0;
  netmask.addr = 0;
  gw.addr = 0;
#else
  IP4_ADDR(&ipaddr, IP_ADDR0, IP_ADDR1, IP_ADDR2, IP_ADDR3);
  IP4_ADDR(&netmask, NETMASK_ADDR0, NETMASK_ADDR1 , NETMASK_ADDR2, NETMASK_ADDR3);
  IP4_ADDR(&gw, GW_ADDR0, GW_ADDR1, GW_ADDR2, GW_ADDR3);
#endif

  /* Add your network interface to the netif_list. */
  netif_add(&netif, &ipaddr, &netmask, &gw, NULL, &ethernetif_init, &ethernet_input);

  /*  Registers the default network interface.*/
  netif_set_default(&netif);
  netif_set_up(&netif);

#if LWIP_DHCP
  dhcp_start (&netif);
#endif

  net_create_timer ();
}


static void net_periodic (uint32_t localtime) {
  static uint32_t TCPTimer = 0;
  static uint32_t ARPTimer = 0;
  static uint32_t LEDTimer = 0;
#if LWIP_DHCP
  static uint32_t DHCPfineTimer   = 0;
  static uint32_t DHCPcoarseTimer = 0;
#endif
  static uint32_t IPaddress = 1;
  static char buff[20];
  uint8_t ip[4];

#if LWIP_TCP
  /* TCP periodic process every 250 ms */
  if (localtime - TCPTimer >= TCP_TMR_INTERVAL) {
    TCPTimer = localtime;
    tcp_tmr();
  }
#endif
  
  /* ARP periodic process every 5s */
  if ((localtime - ARPTimer) >= ARP_TMR_INTERVAL) {
    ARPTimer = localtime;
    etharp_tmr();
  }

#if LWIP_DHCP
  /* Fine DHCP periodic process every 500ms */
  if (localtime - DHCPfineTimer >= DHCP_FINE_TIMER_MSECS) {
    DHCPfineTimer = localtime;
    dhcp_fine_tmr();
  }

  /* DHCP Coarse periodic process every 60s */
  if (localtime - DHCPcoarseTimer >= DHCP_COARSE_TIMER_MSECS) {
    DHCPcoarseTimer = localtime;
    dhcp_coarse_tmr();
  }
#endif

  /* Link check and LED blink process every 500ms */
  if ((localtime - LEDTimer) >= 500) {
    LEDTimer = localtime;
    ethernetif_check_link (&netif);
    if (netif_is_link_up(&netif)) {
      LEDs |= 0x80;
      /* Print IP address on LCD display */
      if (IPaddress != netif.ip_addr.addr) {
        IPaddress = netif.ip_addr.addr;
        ip[0] = (uint8_t)(IPaddress);
        ip[1] = (uint8_t)(IPaddress >> 8);
        ip[2] = (uint8_t)(IPaddress >> 16);
        ip[3] = (uint8_t)(IPaddress >> 24);

        GLCD_DrawString (0, 4*24, "                    ");
        sprintf (buff, "IP: %d.%d.%d.%d",ip[0],ip[1],ip[2],ip[3]);
        GLCD_DrawString (0, 4*24, buff);
      }
    }
    else {
      LEDs &= ~0x80;
      if (IPaddress) {
        IPaddress = 0;
        GLCD_DrawString (0, 4*24, "Link Down...        ");
      }
    }
    LEDs ^= 0x01;
    LED_SetOut (LEDs);
  }
}

/*----------------------------------------------------------------------------
 * Application main thread
 *---------------------------------------------------------------------------*/
void app_main (void *argument) {
  LED_Initialize ();

  GLCD_Initialize         ();
  GLCD_SetBackgroundColor (GLCD_COLOR_BLUE);
  GLCD_SetForegroundColor (GLCD_COLOR_WHITE);
  GLCD_ClearScreen        ();
  GLCD_SetFont            (&GLCD_Font_16x24);
  GLCD_DrawString         (0, 1*24, "RTE and LwIP");
  GLCD_DrawString         (0, 2*24, "HTTP demo example");

  net_init ();
  httpd_init ();

  /* Infinite loop */
  while (1) {
    /* check if any packet received */
    ethernetif_poll (&netif);
    /* handle periodic timers for LwIP */
    net_periodic (LocalTime);
  }
}

int main (void) {
  SystemClock_Config ();
  // System Initialization
  SystemCoreClockUpdate();
#ifdef RTE_Compiler_EventRecorder
  // Initialize and start Event Recorder
  EventRecorderInitialize(EventRecordError, 1U);
#endif
 
  osKernelInitialize();                 // Initialize CMSIS-RTOS
  osThreadNew(app_main, NULL, NULL);    // Create application main thread
  osKernelStart();                      // Start thread execution
  for (;;) {}
}
